local BM_HONOR_ID = "bmhonor_";
function BM_Honor_OnLoad()
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_PVP_KILLS_CHANGED");
	this:RegisterEvent("PLAYER_PVP_RANK_CHANGED");
	
	PLUG_IN_FRAME = "BM_Honor";
	PLUG_IN_NAME = "BM Honor";
	BM_ICON_SIZE = BM_HONOR_ICON_SIZE;
	BM_Plugin(PLUG_IN_FRAME, PLUG_IN_NAME,BM_ICON_SIZE, BM_HONOR_ID);
end

function BM_Honor_OnEvent()
	if (event == "VARIABLES_LOADED") then		
		BM_Honor_initialize();
		return;
	end
	if (event == "PLAYER_ENTERING_WORLD") then
		BM_Honor_OnShow();
	end
	BM_Update_Honor_Text();	
	BM_Update_Tooltip_Text();
end

function BM_Honor_initialize()
	if (BHINFOBAR_CONFIG[BM_PLAYERNAME_REALM][BM_PLUGINVARS].bmhonor_ShowIcon == nil) then
		BHINFOBAR_CONFIG[BM_PLAYERNAME_REALM][BM_PLUGINVARS].bmhonor_ShowIcon = true;
	end
	if (BHINFOBAR_CONFIG[BM_PLAYERNAME_REALM][BM_PLUGINVARS].bmhonor_ShowLabel == nil) then
		BHINFOBAR_CONFIG[BM_PLAYERNAME_REALM][BM_PLUGINVARS].bmhonor_ShowLabel = true;
	end
end

function BM_Honor_OnShow()
	BM_Honor_SetIcon();
end

function BM_Honor_SetIcon()
	local icon = BM_HonorIcon;
	local path = "Interface\\PvPRankBadges\\PvPRank";
	local rankName, rankNumber = GetPVPRankInfo(UnitPVPRank("player"));	
	if (TitanGetVar(BM_HONOR_ID, "ShowIcon")) then
		if (rankNumber > 0) then
			icon:SetTexture(format("%s%02d", path, rankNumber));
			icon:SetWidth(BM_HONOR_ICON_SIZE);
			icon:SetHeight(BM_HONOR_ICON_SIZE);
		end
	else
		icon:SetTexture("");
		icon:SetWidth(1);
	end
end

function BM_Update_Honor_Text()
	local lifetimeRank, sessionHK, sessionDK, yesterdayHK, yesterdayDK, lastweekHK, lastweekDK, lifetimeHK, lifetimeDK, yesterdayContribution, lastweekContribution, lastweekRank = GetInspectHonorData();
	-- Current rank
	local rankName, rankNumber = GetPVPRankInfo(UnitPVPRank("player"));	
	if (not rankName) then
		rankName = NONE;
	end
	-- This session's values
	local sessionHK, sessionDK = GetPVPSessionStats();
	if (TitanGetVar(BM_HONOR_ID, "ShowLabel")) then
		BM_HonorText:SetText(BM_HONOR_LABEL .. format(BM_HONOR_TEXT, rankNumber, sessionHK, sessionDK));
	else
		BM_HonorText:SetText(format(BM_HONOR_TEXT, rankNumber, sessionHK, sessionDK));
	end
end

function BM_Update_Tooltip_Text()
	-- Current rank
	local rankName, rankNumber = GetPVPRankInfo(UnitPVPRank("player"));	
	if (not rankName) then
		rankName = NONE;
	end

	-- This session's values
	local sessionHK, sessionDK = GetPVPSessionStats();
	
	-- Yesterday's values	
	local yesterdayHK, yesterdayDK, yesterdayContribution = GetPVPYesterdayStats();

	-- Last Week's values	
	local lastweekHK, lastweekDK, lastweekContribution, lastweekRank = GetPVPLastWeekStats();
	
	-- Lifetime stats
	local lifetimeHK, lifetimeDK, highestRank = GetPVPLifetimeStats();	
	local highestRankName, highestRankNumber = GetPVPRankInfo(highestRank);
	if ( not highestRankName ) then
		highestRankName = NONE;
	end

	BM_HONOR_TOOLTIP_TEXT = ""..
				RANK..": \t" .. BM_WHITE .. rankName .. " (" .. RANK .. " " .. rankNumber .. ")" .."\n"..
				"\n"..
				HONOR_THIS_SESSION .. "\n" .. BM_FONT_OFF ..
				HONORABLE_KILLS .. ": \t" .. BM_GREEN .. sessionHK .."\n".. BM_FONT_OFF ..
				DISHONORABLE_KILLS .. ": \t" .. BM_RED .. sessionDK .. "\n".. BM_FONT_OFF ..
				"\n"..
				BM_WHITE .. HONOR_YESTERDAY .. "\n" .. BM_FONT_OFF ..
				HONORABLE_KILLS .. ": \t" .. BM_GREEN .. yesterdayHK .. "\n" .. BM_FONT_OFF ..
				DISHONORABLE_KILLS .. ": \t" .. BM_RED .. yesterdayDK .. "\n".. BM_FONT_OFF ..
				HONOR_CONTRIBUTION_POINTS .. ": \t" .. BM_WHITE .. yesterdayContribution .."\n".. 
				"\n"..
				HONOR_LASTWEEK .. "\n" .. BM_FONT_OFF ..
				HONORABLE_KILLS .. ": \t" .. BM_GREEN .. lastweekHK .."\n".. BM_FONT_OFF..
				DISHONORABLE_KILLS .. ": \t" .. BM_RED .. lastweekDK .. "\n".. BM_FONT_OFF..
				HONOR_CONTRIBUTION_POINTS .. ": \t" .. BM_WHITE .. lastweekContribution .."\n".. BM_FONT_OFF ..
				HONOR_STANDING .. ": \t" .. BM_WHITE .. lastweekRank .."\n"..
				"\n"..
				HONOR_LIFETIME .. "\n".. BM_FONT_OFF ..
				HONORABLE_KILLS .. ": \t" .. BM_GREEN .. lifetimeHK .."\n".. BM_FONT_OFF ..
				DISHONORABLE_KILLS .. ": \t" .. BM_RED .. lifetimeDK .. "\n".. BM_FONT_OFF ..
				HONOR_HIGHEST_RANK .. ": \t" .. BM_WHITE  .. highestRankName .. BM_FONT_OFF;

	return
end

-- when the mouse goes over the main frame, this gets called
function BM_Honor_OnEnter()
	GameTooltip:SetOwner(this, "ANCHOR_NONE");
	GameTooltip:SetPoint(BM_TOOLTIP_ANCHOR .. BM_PositonToolTip_RightLeft(), this, BM_TOOLTIP_RELPOINT .. BM_PositonToolTip_RightLeft(), 0, 0);
	-- set the tool tip text
	GameTooltip:SetText(BM_HONOR_TOOLTIP_TITLE);
	BM_Tooltip_AddTooltipText(BM_HONOR_TOOLTIP_TEXT);
	GameTooltip:Show();
end

function BM_Honor_OnLeave()
	-- put the tool tip in the default position
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent);
	GameTooltip:Hide();
end


---**** Options ****---
local BMHonor_CheckButtons = { };
local ctext1 = BM_SHOW_ICON;
local ctext2 = BM_SHOW_LABEL;

BMHonor_CheckButtons[ctext1] = {
	id = 1,
	strVar = "bmhonor_ShowIcon",
	pSlashCommand = "BMHonor_Check1_Opt"
};
BMHonor_CheckButtons[ctext2] = {
	id = 2, 
	strVar = "bmhonor_ShowLabel",
	pSlashCommand = "BMHonor_Check2_Opt"
};

function BMHonor_Check1_Opt(msg)
	TitanSetVar(BM_HONOR_ID, "ShowIcon", msg);
	BM_Honor_SetIcon();
	BM_Update_Honor_Text();
end

function BMHonor_Check2_Opt(msg)
	TitanSetVar(BM_HONOR_ID, "ShowLabel", msg);
	BM_Update_Honor_Text();
end

function BM_Honor_RefreshOptions()
	-- Initial Values
	local button, string, checked;
	-- Setup check buttons
	for key, value in BMHonor_CheckButtons do
		button = getglobal("BMHonor_Check" .. value.id);
		string = getglobal("BMHonor_Check" .. value.id .. "Text");
		checked = nil;
		button.disabled = nil;
		--Check Box
		if (BHINFOBAR_CONFIG[BM_PLAYERNAME_REALM][BM_PLUGINVARS][value.strVar] == true) then
			checked = 1;
		else
			checked = 0;
		end
		button:SetChecked(checked);
		string:SetText(key);
		button.pSlashCommand = getglobal(value.pSlashCommand);
	end
end