--[[

	MonkeySpeed:
	A simple speedometer.
	
	Author: Trentin
	
	Resurected by: Quel

--]]

-- Script array, not saved
MonkeySpeed = {};
MonkeySpeed.m_iDeltaTime = 0;

MonkeySpeed.m_fSpeed = 0.0;
MonkeySpeed.m_fSpeedDist = 0.0;
MonkeySpeed.m_bLoaded = false;
MonkeySpeed.m_bVariablesLoaded = false;
MonkeySpeed.m_strPlayer = "";
MonkeySpeed.m_vCurrPos = {};
MonkeySpeed.m_bCalibrate = false;

local BM_SPEED_ID = "monkeyspeed_";

function MonkeySpeed_Init()
	if (not MonkeySpeedConfig) then
		MonkeySpeedConfig = { };
	end
	if (BHINFOBAR_CONFIG[BM_PLAYERNAME_REALM][BM_PLUGINVARS].monkeyspeed_ShowIcon == nil) then
		BHINFOBAR_CONFIG[BM_PLAYERNAME_REALM][BM_PLUGINVARS].monkeyspeed_ShowIcon = true;
	end
	if (BHINFOBAR_CONFIG[BM_PLAYERNAME_REALM][BM_PLUGINVARS].monkeyspeed_ShowLabel == nil) then
		BHINFOBAR_CONFIG[BM_PLAYERNAME_REALM][BM_PLUGINVARS].monkeyspeed_ShowLabel = true;
	end
	--RegisterForSave("MonkeySpeedConfig");
	-- now we're ready to calculate speed
	MonkeySpeed.m_vLastPos = {};
	MonkeySpeed.m_vLastPos.x, MonkeySpeed.m_vLastPos.y = GetPlayerMapPosition("player");

	if (MonkeySpeedConfig.m_ZoneBaseline1 == nil) then
		MonkeySpeedConfig.m_ZoneBaseline1 = {
					{zid=1, rate=0.00182}, -- Ashenvale
					{zid=2, rate=0.00207}, -- Azshara
					{zid=3, rate=0.00160}, -- Darkshore
					{zid=4, rate=0.00992}, -- Darnassus
					{zid=5, rate=0.00234}, -- Desolace
					{zid=6, rate=0.00199}, -- Durotar
					{zid=7, rate=0.00200}, -- Dustwallow Marsh
					{zid=8, rate=0.00183}, -- Felwood
					{zid=9, rate=0.00151}, -- Feralas
					{zid=10, rate=0.00455}, -- Moonglade
					{zid=11, rate=0.00204}, -- Mulgore
					{zid=12, rate=0.00748}, -- Orgrimmar
					{zid=13, rate=0.00151}, -- Silithus
					{zid=14, rate=0.00215}, -- Stonetalon Mtns
					{zid=15, rate=0.00152}, -- Tanaris
					{zid=16, rate=0.00206}, -- Rut'theran Village
					{zid=17, rate=0.00104}, -- The Barrens
					{zid=18, rate=0.00239}, -- Thousand Needles
					{zid=19, rate=0.01006}, -- Thunderbluff
					{zid=20, rate=0.00284}, -- Un'Goro Crater
					{zid=21, rate=0.00148}, -- Winterspring
					{zid=22, rate=0.0001},
					{zid=23, rate=0.0001},
					{zid=24, rate=0.0001},
					{zid=25, rate=0.0001}
		};
	end

	if (MonkeySpeedConfig.m_ZoneBaseline2 == nil) then
		MonkeySpeedConfig.m_ZoneBaseline2 = {
					{zid=1, rate=0.00375}, -- Alterac Mtns
					{zid=2, rate=0.00292}, -- Arathi Highlands
					{zid=3, rate=0.0042}, -- Badlands
					{zid=4, rate=0.00313}, -- Blasted Lands
					{zid=5, rate=0.00359}, -- Burning Steppes
					{zid=6, rate=0.00420}, -- Deadwind Pass
					{zid=7, rate=0.00213}, -- Dun Morogh
					{zid=8, rate=0.00389}, -- Duskwood
					{zid=9, rate=0.00271}, -- Eastern Plaguelands
					{zid=10, rate=0.00302}, -- Elwynn Forest
					{zid=11, rate=0.00328}, -- Hillsbrad
					{zid=12, rate=0.01327}, -- Ironforge
					{zid=13, rate=0.00381}, -- Loch Modan
					{zid=14, rate=0.00484}, -- Redridge Mtns
					{zid=15, rate=0.00471}, -- Searing Gorge
					{zid=16, rate=0.00250}, -- Silverpine Forest
					{zid=17, rate=0.00781}, -- Stormwind
					{zid=18, rate=0.00165}, -- Stranglethorn Vale
					{zid=19, rate=0.00458}, -- Swamp of Sorrows
					{zid=20, rate=0.00273}, -- Hinterlands
					{zid=21, rate=0.00232}, -- Tirisfal Glades
					{zid=22, rate=0.01094}, -- Undercity
					{zid=23, rate=0.00244}, -- Western Plaguelands
					{zid=24, rate=0.00300}, -- Westfall
					{zid=25, rate=0.00254} -- Wetlands
		};
	end
	
	-- All variables are loaded now
	MonkeySpeed.m_bLoaded = true;
	MonkeySpeed_OnShow();
	
	-- print out a nice message letting the user know the addon loaded
	if (DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage(MONKEYSPEED_LOADED);
	end
end

-- OnLoad Function
function MonkeySpeed_OnLoad()
	-- register events
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("ZONE_CHANGED_NEW_AREA");
	
	-- this command recalibrates the speed calculations for this zone
	SlashCmdList["MONKEYSPEED_CALIBRATE"] = MonkeySpeedSlash_CmdCalibrate;
	SLASH_MONKEYSPEED_CALIBRATE1 = "/monkeyspeedcalibrate";
	SLASH_MONKEYSPEED_CALIBRATE2 = "/mscalibrate";

	PLUG_IN_FRAME = "BM_Speed";
	PLUG_IN_NAME = "MonkeySpeed";
	BM_ICON_SIZE = BM_MONKEY_ICON_SIZE;
	BM_Plugin(PLUG_IN_FRAME, PLUG_IN_NAME,BM_ICON_SIZE, BM_SPEED_ID);
end

-- OnEvent Function
function MonkeySpeed_OnEvent(event)
	if (event == "VARIABLES_LOADED") then
		MonkeySpeed_Init();
	elseif (event == "ZONE_CHANGED_NEW_AREA") then
		SetMapToCurrentZone();
	end
end

-- OnUpdate Function (heavily based off code in Telo's Clock)
function MonkeySpeed_OnUpdate(arg1)
	local buttonText = "";
	-- if the speedometer's not loaded yet, just exit
	if (not MonkeySpeed.m_bLoaded) then
		return;
	end
	
	-- how long since the last update?
	MonkeySpeed.m_iDeltaTime = MonkeySpeed.m_iDeltaTime + arg1;
	
	-- update the speed calculation
	MonkeySpeed.m_vCurrPos.x, MonkeySpeed.m_vCurrPos.y = GetPlayerMapPosition("player");
	MonkeySpeed.m_vCurrPos.x = MonkeySpeed.m_vCurrPos.x + 0.0;
	MonkeySpeed.m_vCurrPos.y = MonkeySpeed.m_vCurrPos.y + 0.0;

	if (MonkeySpeed.m_vCurrPos.x) then
		local dist;
		
		-- travel speed ignores Z-distance (i.e. you run faster up or down hills)	
		-- x and y coords are not square, had to weight the x by 2.25 to make the readings match the y axis.
		dist = math.sqrt(
				((MonkeySpeed.m_vLastPos.x - MonkeySpeed.m_vCurrPos.x) * (MonkeySpeed.m_vLastPos.x - MonkeySpeed.m_vCurrPos.x) * 2.25 ) +
				((MonkeySpeed.m_vLastPos.y - MonkeySpeed.m_vCurrPos.y) * (MonkeySpeed.m_vLastPos.y - MonkeySpeed.m_vCurrPos.y)));
		
		MonkeySpeed.m_fSpeedDist = MonkeySpeed.m_fSpeedDist + dist;
		if (MonkeySpeed.m_iDeltaTime >= .5) then
			-- The map coords seem to be a different scale in different zones. Figure out which zone we're in
			local zonenum;
			local contnum;
			local baserate;

			zonenum = GetCurrentMapZone();
			if (zonenum ~= 0) then
				contnum = GetCurrentMapContinent();
				f,h,w = GetMapInfo();


				if (MonkeySpeed.m_bCalibrate == true) then
					-- recalibrate this zone, the user should know this should be done when running at 100%
					if (contnum == 2) then
						MonkeySpeedConfig.m_ZoneBaseline2[zonenum].rate = MonkeySpeed.m_fSpeedDist / MonkeySpeed.m_iDeltaTime;
					else
						MonkeySpeedConfig.m_ZoneBaseline1[zonenum].rate = MonkeySpeed.m_fSpeedDist / MonkeySpeed.m_iDeltaTime;
					end

					-- done calibrating
					MonkeySpeed.m_bCalibrate = false;
				end


				if (contnum == 2) then
					baserate = MonkeySpeedConfig.m_ZoneBaseline2[zonenum].rate;
				else
					baserate = MonkeySpeedConfig.m_ZoneBaseline1[zonenum].rate;
				end
				
				MonkeySpeed.m_fSpeed = MonkeySpeed_Round(((MonkeySpeed.m_fSpeedDist / MonkeySpeed.m_iDeltaTime) / baserate) * 100);
			
				MonkeySpeed.m_fSpeedDist = 0.0;
				MonkeySpeed.m_iDeltaTime = 0.0;
			
				-- Set the text for the speedometer
				if (MonkeySpeed.m_fSpeed < 50) then
					buttonText = format(MONKEYSPEED_TEXT_RED, MonkeySpeed.m_fSpeed);
				elseif (MonkeySpeed.m_fSpeed > 50 and MonkeySpeed.m_fSpeed < 85) then
					buttonText = format(MONKEYSPEED_TEXT_YELLOW, MonkeySpeed.m_fSpeed);
				else
					buttonText = format(MONKEYSPEED_TEXT_GREEN, MonkeySpeed.m_fSpeed);
				end
				if (TitanGetVar(BM_SPEED_ID, "ShowLabel")) then
					BM_SpeedText:SetText(BM_SPEED_LABEL .. buttonText);
				else
					BM_SpeedText:SetText(buttonText);
				end
			end
		end

		
		MonkeySpeed.m_vLastPos.x = MonkeySpeed.m_vCurrPos.x;
		MonkeySpeed.m_vLastPos.y = MonkeySpeed.m_vCurrPos.y;
		MonkeySpeed.m_vLastPos.z = MonkeySpeed.m_vCurrPos.z;
	end
end

-- when the mouse goes over the main frame, this gets called
function MonkeySpeed_OnEnter()
	-- put the tool tip in the default position
	GameTooltip:SetOwner(this, "ANCHOR_NONE");
	GameTooltip:SetPoint(BM_TOOLTIP_ANCHOR .. BM_PositonToolTip_RightLeft(), this, BM_TOOLTIP_RELPOINT .. BM_PositonToolTip_RightLeft(), 0, 0);
	-- set the tool tip text
	GameTooltip:SetText(MONKEYSPEED_TITLE_VERSION, ITEM_QUALITY6_TOOLTIP_COLOR.r, ITEM_QUALITY6_TOOLTIP_COLOR.g, ITEM_QUALITY6_TOOLTIP_COLOR.b, 1);
	GameTooltip:AddLine(MONKEYSPEED_DESCRIPTION, GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b, 1);
	GameTooltip:Show();
end

function MonkeySpeed_OnLeave()
	-- put the tool tip in the default position
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent);
	GameTooltip:Hide();
end

function MonkeySpeed_ParsePosition(position)
	local x, y, z;
	local iStart, iEnd;

	iStart, iEnd, x, y = string.find(position, "^(.-), (.-)$");

	if( x ) then
		return x + 0.0, y + 0.0;
	end
	return nil, nil;
end

function MonkeySpeed_Round(x)
	if(x - floor(x) > 0.5) then
		x = x + 0.5;
	end
	return floor(x);
end

function MonkeySpeed_OnShow()
	MonkeySpeed_SetIcon();
end

function MonkeySpeed_SetIcon()
	local icon = BM_SpeedIcon;
	if (TitanGetVar(BM_SPEED_ID, "ShowIcon")) then
		icon:SetTexture(BM_IMAGE_PATH .. "Icon_speed");
		icon:SetWidth(BM_MONKEY_ICON_SIZE);
		icon:SetHeight(BM_MONKEY_ICON_SIZE);
	else
		icon:SetTexture("");
		icon:SetWidth(1);
	end
end

---**** Options ****---
function MonkeySpeed_calibarate()
	MonkeySpeed.m_bCalibrate = true;
end

function BM_Speed_RefreshOptions()
	-- No Options to refresh at this time
end

function MonkeySpeed_calibarate_Tooltip_Show()
	-- put the tool tip in the default position
	GameTooltip:SetOwner(this, "ANCHOR_NONE");
	GameTooltip:SetPoint("BOTTOMLEFT", this, "TOPRIGHT", 0, 0);
	-- set the tool tip text
	GameTooltip:SetText(MONKEYSPEED_TITLE_VERSION, ITEM_QUALITY6_TOOLTIP_COLOR.r, ITEM_QUALITY6_TOOLTIP_COLOR.g, ITEM_QUALITY6_TOOLTIP_COLOR.b, 1);
	GameTooltip:AddLine(MONKEYSPEED_CALIBRATE_TOOLTIP, 1, 1, 1, 1);
	GameTooltip:Show();
end

function MonkeySpeed_calibarate_Tooltip_Reset()
	-- put the tool tip in the default position
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent);
	GameTooltip:Hide();
end

---**** Options ****---
local MonkeySpeed_CheckButtons = { };
local ctext1 = BM_SHOW_ICON;
local ctext2 = BM_SHOW_LABEL;

MonkeySpeed_CheckButtons[ctext1] = {
	id = 1,
	strVar = "monkeyspeed_ShowIcon",
	pSlashCommand = "monkeyspeed_Check1_Opt"
};
MonkeySpeed_CheckButtons[ctext2] = {
	id = 2, 
	strVar = "monkeyspeed_ShowLabel",
	pSlashCommand = "monkeyspeed_Check2_Opt"
};

function monkeyspeed_Check1_Opt(msg)
	TitanSetVar(BM_SPEED_ID, "ShowIcon", msg);
	MonkeySpeed_SetIcon();
end

function monkeyspeed_Check2_Opt(msg)
	TitanSetVar(BM_SPEED_ID, "ShowLabel", msg);
end

function BM_Speed_RefreshOptions()
	-- Initial Values
	local button, string, checked;
	-- Setup check buttons
	for key, value in MonkeySpeed_CheckButtons do
		button = getglobal("BM_Speed_Check" .. value.id);
		string = getglobal("BM_Speed_Check" .. value.id .. "Text");
		checked = nil;
		button.disabled = nil;
		--Check Box
		if (BHINFOBAR_CONFIG[BM_PLAYERNAME_REALM][BM_PLUGINVARS][value.strVar] == true) then
			checked = 1;
		else
			checked = 0;
		end
		button:SetChecked(checked);
		string:SetText(key);
		button.pSlashCommand = getglobal(value.pSlashCommand);
	end
end